import 'package:flutter/cupertino.dart';
import 'package:go_router/go_router.dart';
import 'package:starting_template/core/enums/page_protection.dart';
import 'package:starting_template/core/extensions/go_route_guard.dart';
import 'package:starting_template/core/extensions/route_base_extension.dart';
import 'package:starting_template/core/routes/go_router_refresh_stream.dart';
import 'package:starting_template/presentation/features/home/home.dart';

abstract class AppRouter {
  /// Default transition for all pages
  static Page<void> defaultTransition(
    BuildContext context,
    GoRouterState state,
    Widget child,
  ) =>
      CupertinoPage<void>(key: state.pageKey, child: child);

  /// Disable transition animation
  static Page<void> noTransition(
    BuildContext context,
    GoRouterState state,
    Widget child,
  ) =>
      NoTransitionPage(key: state.pageKey, child: child);

  /// Defines GoRoute routes.
  static final List<GoRoute> routes = [
    GoRoute(
      path: '/',
      name: Home.pageName,
      pageBuilder: (context, state) =>
          defaultTransition(context, state, const Home()),
    )..setPublic(),
  ];

  /// Singleton instance of [GoRouter]
  static GoRouter? _router;

  /// Navigator key for the root navigator
  static final GlobalKey<NavigatorState> rootNavigatorKey =
      GlobalKey<NavigatorState>();

  /// Router
  static GoRouter routerOf(BuildContext context) => _router ??= GoRouter(
        initialLocation: '/',
        routes: AppRouter.routes,
        navigatorKey: rootNavigatorKey,
        debugLogDiagnostics: true,
        refreshListenable: GoRouterRefreshStream(const Stream.empty()),
        // TODO(wyatt): Add authentication logic
        redirect: (context, state) {
          /// Define the default guard
          /// This is the guard that will be used if the route
          /// does not have a guard set. (It is set to [PageProtection.none])
          const defaultGuard = PageProtection.protected;

          /// Recursively get all routes and subroutes in a one-dimensional list
          /// This will work for any level of nesting
          final routes =
              AppRouter.routes.map((route) => route.flatten()).expand(
                    (routeList) => routeList,
                  );

          /// Matches is a list of all routes that match the current uri
          final matches = _router?.configuration
                  .findMatch(
                    state.uri.toString(),
                  )
                  .routes
                  .map((route) => route.flatten())
                  .expand(
                    (routeList) => routeList,
                  ) ??
              routes;

          /// Get the current route
          final currentRoute = matches.last;

          /// Get the guard of the current route
          final guard = (currentRoute.guard == PageProtection.none)
              ? defaultGuard
              : currentRoute.guard;

          return null;
        },
      );
}
